/*
TSC - A program to convert Truevision Targa (TGA) and bitmap (BMP) images to data for use in TI assembly or C programming
Copyright (C) 2007  Peter Fernandes

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#include "main.h"


int load_image(IMAGE *img, FILE *src, options *opts) {
	unsigned char *ptr;
	unsigned char *ptr2;
	int temp=0;
	int h=0;
	int color=0;
	char *flipbuf=NULL;
	
	
	fread(&temp,1,2,src);
	
	
	if(temp==((int)'M'<<8)+'B') { //if we have a bitmap file
		if(!(flipbuf=malloc(img->width))) {
			return -3;
		}
		
		fseek(src,18,SEEK_SET);
		fread(&img->width,4,1,src);
		fread(&img->height,4,1,src);
		
		fseek(src,28,SEEK_SET);
		
		fread(&temp,2,1,src);
		
		if(temp!=24) {
			if(temp)
				free(flipbuf);
			return -2;
		}
		
		fseek(src,10,SEEK_SET);
		
		fread(&temp,4,1,src);
		
		fseek(src,temp,SEEK_SET);
		temp=1;
	}
	else { //else try for TGA
		fseek(src,12,SEEK_SET);

		img->width=fgetc(src);
		img->width+=fgetc(src)<<8;
		img->height=fgetc(src);
		img->height+=fgetc(src)<<8;
	
	
		if(fgetc(src)!=24) {
			return -2;
		}
	
			
		fseek(src,18,SEEK_SET);
		temp=0;
		h=0;
	}
	
	if(img->width&7) {
		if(temp)
			free(flipbuf);
		return -4;
	}

	if(!(img->data=malloc(img->width*img->height))) {
		if(temp)
			free(flipbuf);
		return -3;
	}
	
	ptr=img->data;
	
		
	//get color data
	for(h=0;h<(img->height*img->width);h++) {
		color=((fgetc(src)+fgetc(src)+fgetc(src))/3)+((double)opts->brightness*5.12);
		color=move_to_val(128,color,-((double)(opts->contrast-50)*2.56));
		
		if(color>255) color=255;
		if(color<0) color=0;
		

		switch (opts->shades) {
			case 2:
				if(color>=127)
					*(ptr++)=WHITE;
				else
					*(ptr++)=BLACK;
				break;
			case 4:
				if(color>=63 && color<126)
					*(ptr++)=DARK;
				else if(color>=126 && color<189)
					*(ptr++)=LIGHT;
				else if(color>=189)
					*(ptr++)=WHITE;
				else
					*(ptr++)=BLACK;
				break;
			case 7:
				if(color>=36 && color<72)
					*(ptr++)=LIGHT_BLACK;
				else if(color>=72 && color<108)
					*(ptr++)=DARK;
				else if(color>=108 && color<144)
					*(ptr++)=LIGHT;
				else if(color>=144 && color<180)
					*(ptr++)=LIGHT_LIGHT;
				else if(color>=180 && color<216)
					*(ptr++)=DARK_WHITE;
				else if(color>=216 && color<=255)
					*(ptr++)=WHITE;
				else
					*(ptr++)=BLACK;
				break;
			case 8:
				if(color>=32 && color<64)
					*(ptr++)=LIGHT_BLACK;
				else if(color>=64 && color<96)
					*(ptr++)=DARK;
				else if(color>=96 && color<128)
					*(ptr++)=LIGHT_DARK;
				else if(color>=128 && color<160)
					*(ptr++)=LIGHT;
				else if(color>=160 && color<192)
					*(ptr++)=LIGHT_LIGHT;
				else if(color>=192 && color<224)
					*(ptr++)=DARK_WHITE;
				else if(color>=224 && color<=255)
					*(ptr++)=WHITE;
				else
					*(ptr++)=BLACK;
		}
	}
	
	//flip if it's a bitmap :(
	if(temp) {
		ptr=img->data;
		ptr2=img->data+((img->height-1)*img->width);
	
		for(h=0;h<(img->height/2);h++) {
			memcpy(flipbuf,ptr2,img->width);
			memcpy(ptr2,ptr,img->width);
			memcpy(ptr,flipbuf,img->width);
			ptr+=img->width;
			ptr2-=img->width;
		}
		free(flipbuf);
	}

		
	
	return 0;
}


char *basename(char *name) {
	char *pos=name+strlen(name)-1;
	char *buffer;

	for(;*pos!='/' && *pos!='\\' && pos>name;pos--);
	
	if(!(buffer=malloc(strlen(pos))))
		return NULL;
	
	strncpy(buffer,pos+(pos==name?0:1),strlen(pos));
	return buffer;
}


int write_preview(char *prgm, unsigned char *buffer, int height, int width, int shades) {
	int i;
	FILE *src;
	char *dirname;
	char space[9];
	char dir[20];
	char filename[32];
	char fnbuf[33+strlen(prgm)];
	
	

	memset(fnbuf,0,sizeof(fnbuf));
	
	memcpy(dir,"/tmp/sprtconvXXXXXX",20);
		
	dirname=mkdtemp(dir);
	
	memcpy(filename,dirname,20);
	
	strcat(filename,"/preview.tga");

	src=fopen(filename,"wb");
	
	if(!src) {
		fprintf(stderr,"Error: Unable to write preview image");
		return -1;
	}

	memset(space,0,9);
	
	fputc(0,src);
	fputc(0,src);
	fputc(2,src);
	fwrite(space,1,9,src);
	
	fputc((char)width,src);
	fputc((char)width>>8,src);
	
	fputc((char)height,src);
	fputc((char)height>>8,src);
	
	fputc(24,src);
	fputc(32,src);
	
	
	for(i=0;i<height*width;i++) {
		if(shades!=7) {
			switch (buffer[i]) {
				case WHITE:
					fputc(255,src);
					fputc(255,src);
					fputc(255,src);
					break;
				case DARK_WHITE:
					fputc(208,src);
					fputc(208,src);
					fputc(208,src);
					break;
				case LIGHT_LIGHT:
					fputc(176,src);
					fputc(176,src);
					fputc(176,src);
					break;
				case LIGHT:
					fputc(144,src);
					fputc(144,src);
					fputc(144,src);
					break;
				case LIGHT_DARK:
					fputc(112,src);
					fputc(112,src);
					fputc(112,src);
					break;
				case DARK:
					fputc(80,src);
					fputc(80,src);
					fputc(80,src);
					break;
				case LIGHT_BLACK:
					fputc(48,src);
					fputc(48,src);
					fputc(48,src);
					break;
				case BLACK:
					fputc(0,src);
					fputc(0,src);
					fputc(0,src);
					break;
				default:
					return -1;
			}
		}
		else {
			switch (buffer[i]) {
				case WHITE:
					fputc(255,src);
					fputc(255,src);
					fputc(255,src);
					break;
				case DARK_WHITE:
					fputc(198,src);
					fputc(198,src);
					fputc(198,src);
					break;
				case LIGHT_LIGHT:
					fputc(162,src);
					fputc(162,src);
					fputc(162,src);
					break;
				case LIGHT:
					fputc(126,src);
					fputc(126,src);
					fputc(126,src);
					break;
				case DARK:
					fputc(90,src);
					fputc(90,src);
					fputc(90,src);
					break;
				case LIGHT_BLACK:
					fputc(54,src);
					fputc(54,src);
					fputc(54,src);
					break;
				case BLACK:
					fputc(0,src);
					fputc(0,src);
					fputc(0,src);
					break;
				default:
					return -1;
			}
		
		
		}
	}

	memcpy(fnbuf,prgm,strlen(prgm));
	strcat(fnbuf," ");
	strcat(fnbuf,filename);

	fclose(src);
	
	system(fnbuf);

	return 0;
}

